#ifndef E24_H
#define E24_H

#ifdef _WIN32
  #define FILE_PATH "C:/dev/e24/"
#else
  #define FILE_PATH "/home/dev/e24/"
#endif

//calibrations
#define E24CAL_NONE   0
#define E24CAL_AUTO   1
#define E24CAL_EXT0   2
#define E24CAL_EXTA   3
#define E24CAL_FULL   4
#define E24CAL_BG     5
#define E24CAL_INT0   6
#define E24CAL_INTA   7

//amplificators
#define E24AMP_1      0
#define E24AMP_2      1
#define E24AMP_4      2
#define E24AMP_8      3
#define E24AMP_16     4
#define E24AMP_32     5
#define E24AMP_64     6
#define E24AMP_128    7

//configure results
#define E24RES
#define E24RES_OK         0
#define E24RES_DISCONNECT 1
#define E24RES_CHAN       2
#define E24RES_FREQ       3
#define E24RES_RANGE      4
#define E24RES_CAL        5
#define E24RES_SUBCHAN    6
#define E24RES_APPLY      7

/************************************************************************************************
 * Функции, реализованные в библиотеке e24hid.so / e24hid.dll
 ************************************************************************************************
 */

/* Переинициализация АЦП e24 и установка настроек по умолчанию
 * 
 * Входные параметры:
 *   нет
 * Результат:
 *   0 - успех
 *   1 - ошибка связи
 */
char e24_reinit(void);
/* Конфигурация одного канала
 * 
 * Входные параметры:
 *   chan - номер канала
 *   freq - частота (отсчетов в секунду)
 *   adc_range - максимальное напряжение на канале (В). Будет подобрано ближайшее значение из доступных
 *   calibrate - калибровка:
 *     0 - нет
 *     1 - самокалибровка (default)
 *     2 - калибровка нуля
 *     3 - калибровка масштаба
 *     4 - смешанная калибровка
 *     5 - фоновая
 *     6 - внутренняя калибровка нуля
 *     7 - внутренняя калибровка масштаба
 *   subchannel - мультиплексор канала
 *     0 - канал A
 *     1 - канал B
 *     2 - заземление
 *     3 - тестовый режим
 * Результат:
 *   0 - успех
 *   1 - ошибка связи
 *   2 - неверно задан канал
 *   3 - невозможно передать частоту (ошибка связи)
 *   4 - невозможно передать диапазон (ошибка связи)
 *   5 - невозможно передать калибровку (ошибка связи)
 *   6 - невозможно передать subchannel (ошибка связи)
 *   7 - невозможно применить изменения (ошибка связи)
 */
char e24_configure(uint8_t chan, uint16_t freq, float adc_range, uint8_t calibrate, uint8_t subchannel);
/* Чтение выбранного канала с драйвера. Не напрямую с устройства!
 * 
 * Входные параметры:
 *   номер канала (1 - 4)
 * Результат:
 *   напряжение на канале (в вольтах) либо ошибка (1e10 В - в штатном режиме такое значение невозможно)
 */
double e24_read(uint8_t chan);
/*   Вывод подсказки. В основном для LabView. Поскольку там нет нормального вывода в консоль, приходится
 * возвращать строку.
 * 
 * Входные параметры:
 *   нет
 * Результат:
 *   строка с кратким описанием функций
 */
char* e24_help(void);

#endif
