/*   Драйвер работы с e24 (исполняемый файл). Разрабатывался в первую очередь для win32,
 * но заработает и под Linux'ом. Хотя там можно было сделать проще.
 *   Драйвер должен быть запущен в фоновом режиме, а остановка осуществляется сигналами SIGINT и SIGTERM.
 * Устройство может быть подключено как после запуска драйвера, так и до него. В частности, можно
 * добавить в автозапуск.
 *   Драйвер отображает значения АЦП (в вольтах) в файлы.
 *   Формат: число с плавающей точкой и, через табуляцию, время в миллисекундах от запуска драйвера до последнего
 * обновления для проверки достоверности данных.
 */
#include <stdio.h>
#include <inttypes.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <signal.h>
#include "vhid.h"
#include "memfile.h"
#include "daemon.h"
#include "../../firmware/commands.h"
#include "e24hid.h"

hiddevice_t dev;
struct dataexchange_t pdata;
struct memfile_t adc_file[4];

unsigned long time_ms(){
  struct timeval t;
  gettimeofday(&t, 0);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

int main(int argc, char **argv){
  char data[10];
  unsigned char res;
  double adc[4] = {0,0,0,0};
  int32_t adc_code;
  int hid_num;
  unsigned long curtime, oldtime=0;

  hid_num = dmn_init_std(argc, argv, NULL);
  if(hid_num != DMN_DAEMON)return hid_num; //в случае ошибки выходим с соответствующим кодом возврата
  
  if(memfile_create(FILE_PATH"adc1.adc", MEMFILE_READWRITE | MEMFILE_REWRITE, 100, &adc_file[0]) != MEMFILE_OK){
    dmn_log("Error %i create file %s\n",hid_num,FILE_PATH"adc1.adc"); return -50;
  }
  if(memfile_create(FILE_PATH"adc2.adc", MEMFILE_READWRITE | MEMFILE_REWRITE, 100, &adc_file[1]) != MEMFILE_OK){
    dmn_log("Error %i create file %s\n",hid_num,FILE_PATH"adc2.adc"); return -50;
  }
  if(memfile_create(FILE_PATH"adc3.adc", MEMFILE_READWRITE | MEMFILE_REWRITE, 100, &adc_file[2]) != MEMFILE_OK){
    dmn_log("Error %i create file %s\n",hid_num,FILE_PATH"adc3.adc"); return -50;
  }
  if(memfile_create(FILE_PATH"adc4.adc", MEMFILE_READWRITE | MEMFILE_REWRITE, 100, &adc_file[3]) != MEMFILE_OK){
    dmn_log("Error %i create file %s\n",hid_num,FILE_PATH"adc4.adc"); return -50;
  }

  dev = HidOpenDevice(0x16C0, 0x05DF, L"COKPOWEHEU", L"e24 HID");

  oldtime = time_ms();
  while(1){
    curtime = time_ms();

    if(HidIsConnected(dev)){
      hid_num = HidReceiveArr(dev, data, 4);
      if(hid_num == 4){
        uint8_t scale = 1;
        if((data[0] & REP_3BIT_MASK)==REP_ADC){
          //объединение отдельных байтов в 24-битное значение
          adc_code = (((uint32_t)(unsigned char)data[1])<<16) | (((uint32_t)(unsigned char)data[2])<<8) | ((uint32_t)(unsigned char)data[3]);
          //перевод в знаковую запись (в e24 нулем считается 0x00800000)
          adc_code &= 0x00FFFFFF;
          adc_code -= 0x00800000;
          //номер канала - 2 младших бита)
          res = data[0]&0x03;
          //диапазон измерения - биты 2,3,4
          scale = (data[0]>>2)&0x07;
          //перевод значения АЦП в вольты
          adc[res] = adc_code;
          adc[res] *= 2.5 / 0x007FFFFF;
          while(scale--){
            adc[res] /= 2;
          }
          //вывод в отображаемые в память файлы
          if(adc_file[res].data != NULL)snprintf(adc_file[res].data, 100, "%.11e\t%lu\t", adc[res],curtime-oldtime);
        }
      }else{//if(hid_num != 4)
        //если произошел сбой, отключаемся
        HidCloseDevice(dev);
      }
    }else{//if(HidIsConnected)
      //если устройство не подключено - пытаемся подключиться
      dev = HidOpenDevice(0x16C0, 0x05DF, L"COKPOWEHEU", L"e24 HID");
    }
    if(!dmn_running())break;
    //период обновления 10 мс - все равно low-speed usb больше не позволяет
    usleep(10000);
  }
  //перед завершением работы очищаем файлы
  if(adc_file[0].data != NULL)memset(adc_file[0].data, ' ', 100);
  if(adc_file[1].data != NULL)memset(adc_file[1].data, ' ', 100);
  if(adc_file[2].data != NULL)memset(adc_file[2].data, ' ', 100);
  if(adc_file[3].data != NULL)memset(adc_file[3].data, ' ', 100);
  
  dmn_close();
  return 0;
}

__attribute__((__destructor__))void del(){
  HidCloseDevice(dev);
  memfile_close(&adc_file[0]);
  memfile_close(&adc_file[1]);
  memfile_close(&adc_file[2]);
  memfile_close(&adc_file[3]);
}
#ifdef _WIN32
//без этого (и модуляей вроде libgdi32.a) получается консольное приложение, а окно консоли для данной задачи не нужно
#include "windows.h"
WINAPI int WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow){
  return main(0, NULL);
}
#endif
